/*

http://www.lingoworkshop.com/Articles/Isometric_Game_1.php
http://pgdc.purdue.org/sdltutorial/01/
http://docs.mandragor.org/files/Common_libs_documentation/SDL/SDL_Documentation_project_en/

gcc -lSDL -lSDL_gfx -lSDL_image main.c
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <termios.h> /* POSIX terminal control definitions */
#include <fcntl.h>   /* File control definitions */

#include <SDL/SDL.h>
#include <SDL/SDL_gfxPrimitives.h>
#include <SDL/SDL_image.h>

#include "IntList.h"

#define WIDTH  640
#define HEIGHT 480

#define numtiles     6

#define numcols      20
#define numrows      20
#define Tile_HalfW   20
#define Tile_HalfH   12
#define tileOffsetY  0
#define tileOffsetX  320
#define hOffset      0
#define vOffset      0


void DrawScene(SDL_Surface *screen);

void Init(void);
void Slock(SDL_Surface *screen) ;
void Sulock(SDL_Surface *screen) ;

SDL_Surface  *screen;
SDL_Surface  **tiles;
unsigned int **field;

#define limit(v, l, h)  ((v) > (h)) ? (h) : ((v) < (l)) ? (l) : (v)






int main(int argc, char *argv[]) {

  int done;
  char temp[64];
  int i, j;
  int x, y;
  
  SDL_Event event;

  Init();
  
  done = 0;
  
  // allocate tiles
  if ((tiles = malloc(sizeof(SDL_Surface *)*numtiles)) != NULL) {
    for(i = 0; i < numtiles; i++) {
      sprintf(temp, "tile%d.png", i);
      tiles[i] = IMG_Load(temp);
    }
  } 
  
  // allocate and assign field tiles
  if ((field = malloc(sizeof(int *)*numrows)) != NULL) {
    for(x = 0; x < numrows; x++) {
      if ((field[x] = malloc(sizeof(int)*numcols)) != NULL) {
        for(y = 0; y < numcols; y++) {
          field[x][y] = 0;
        }  
      }
    }
  } 
  
  field[0][0] = 1;
  
  field[6][8] = 2;
  field[6][9] = 2;
  field[7][9] = 3;
  field[8][9] = 3;
  field[9][9] = 4;
  field[9][8] = 4;
  field[8][8] = 5;
  field[7][8] = 5;
  
  while(done == 0) {   
    while ( SDL_PollEvent(&event) ) {
    
      if ( event.type == SDL_QUIT )  {  done = 1;  }     

      if ( event.type == SDL_KEYDOWN ) {
        if ( event.key.keysym.sym == SDLK_ESCAPE ) { done = 1; }              
      }
      
      SDL_FillRect(screen, NULL, 0);                   
      DrawScene(screen);  
    } 
    
    // limit refresh to 50Hz, for serial xmitt
    usleep(1); 
 }




  for(i = 0; i < numtiles; i++) {
    SDL_FreeSurface(tiles[i]);
  }
    
  return 0;
}



/****************************/

void DrawScene(SDL_Surface *screen) {
 int x, y, wx, wy;
 SDL_Rect pictureLocation;

  Slock(screen);
  
  for (y = 0; y < numcols; y++) {
    for (x = 0; x < numrows; x++) {
      wx = (x * Tile_HalfW + tileOffsetX) - (y * Tile_HalfW) + hOffset;
      wy = (x * Tile_HalfH - tileOffsetY) + (y * Tile_HalfH) + vOffset;
      pictureLocation.x = wx;
      pictureLocation.y = wy;
      SDL_BlitSurface(tiles[field[x][y]], NULL, screen, &pictureLocation);
    }
  }
  
  
 /*
  rectangleRGBA (screen, 0, 0, 256, 256, 250, 250, 250, 255 ); // navigation box
  stringRGBA(screen, 257, offset+1*(space+lineheight), " 4800N81",     0, 0, 255, 255); // foregroud colour
  circleRGBA (screen, 128, 128, x-1,  ((255-x)*2)-1, (x*2)-1, 0, 255 );
  lineRGBA (screen, px,    py-10, px,    py+10, 0, 0, 255, 255 );
*/
  
  
  Sulock(screen);
  SDL_Flip(screen);
  
}

/**************************************************************************/

void Init(void) {

  if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
    printf("Unable to init SDL: %s\n", SDL_GetError());
    exit(1);
  }
  atexit(SDL_Quit);

  
  screen=SDL_SetVideoMode(WIDTH, HEIGHT, 32, SDL_HWSURFACE|SDL_DOUBLEBUF);
  
  if ( screen == NULL ) {
    printf("Unable to set 256x256 video: %s\n", SDL_GetError());
    exit(1);
  }
  
  //turn off cursor...
  //SDL_ShowCursor(0);
}


void Slock(SDL_Surface *screen) {
  if ( SDL_MUSTLOCK(screen) ) {
    if ( SDL_LockSurface(screen) < 0 ) {
      return;
    }
  }
}

void Sulock(SDL_Surface *screen) {
  if ( SDL_MUSTLOCK(screen) ) {
    SDL_UnlockSurface(screen);
  }
}








