/*

http://www.lingoworkshop.com/Articles/Isometric_Game_1.php
http://pgdc.purdue.org/sdltutorial/01/
http://docs.mandragor.org/files/Common_libs_documentation/SDL/SDL_Documentation_project_en/

gcc -lSDL -lSDL_gfx -lSDL_image main.c IntList.c

Y</////o\\\\\>X

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <termios.h> /* POSIX terminal control definitions */
#include <fcntl.h>   /* File control definitions */

#include <SDL/SDL.h>
#include <SDL/SDL_gfxPrimitives.h>
#include <SDL/SDL_image.h>

#include "IntList.h"

#define WIDTH  640
#define HEIGHT 480

#define numtiles     9
#define numgroups    10

#define numcols      20
#define numrows      20
#define Tile_HalfW   20
#define Tile_HalfH   12

//#define numcols      10
//#define numrows      10
//#define Tile_HalfW   40
//#define Tile_HalfH   24

#define tileOffsetY  0
#define tileOffsetX  320
#define hOffset      0
#define vOffset      0


void DrawScene(SDL_Surface *screen);

void Init(void);
void Slock(SDL_Surface *screen) ;
void Sulock(SDL_Surface *screen) ;

SDL_Surface  *screen;
SDL_Surface  **tiles;
IntList_t    ***field;
IntList_t    **groups;
IntList_t    *animate;

#define limit(v, l, h)  ((v) > (h)) ? (h) : ((v) < (l)) ? (l) : (v)

#define rnd() ( rand() / (RAND_MAX + 1.0) )




int main(int argc, char *argv[]) {

  int done;
  char temp[64];
  int i, j;
  int x, y;
  
  SDL_Event event;

  Init();
  
  done = 0;
  
  
  // allocate tiles
  if ((tiles = malloc(sizeof(SDL_Surface *)*numtiles)) != NULL) {
    for(i = 0; i < numtiles; i++) {
      sprintf(temp, "tile%d.png", i);
      tiles[i] = IMG_Load(temp);
    }
  } 
  
  
  // allocate grouping arrays
  if ((groups = malloc(sizeof(IntList_t **)*numgroups)) != NULL) {
    for(i = 0; i < numgroups; i++) {
      groups[i] =  IntList_init();
      IntList_add(groups[i], i);
    }  
  }
  
  IntList_del(groups[9], 0);
  IntList_add(groups[9], 2);
  IntList_add(groups[9], 3);
  IntList_add(groups[9], 4);
  IntList_add(groups[9], 5);
  
  
  // allocate and assign field tiles
  if ((field = malloc(sizeof(IntList_t **)*numrows)) != NULL) {
    for(x = 0; x < numrows; x++) {
      if ((field[x] = malloc(sizeof(IntList_t *)*numcols)) != NULL) {
        for(y = 0; y < numcols; y++) {
          field[x][y] =  IntList_init();
          IntList_add(field[x][y], 0); // 0 is a group reference
        }  
      }
    }
  } 
  
  
  
  // tree patch
  for (x = 1; x < 10; x++) for (y = 0; y < 8; y++) {
   
    if (rnd() > 0.8) {
      IntList_add(field[x][y], 7);
    } else if (rnd() > 0.8){
      IntList_add(field[x][y], 8);
    } 
    
  }
  
  // penn
  IntList_add(field[6][8], 2);
  IntList_add(field[6][9], 2);
  IntList_add(field[6][9], 2);
  
  IntList_add(field[7][9], 3);
  IntList_add(field[8][9], 3);
//  IntList_add(field[9][9], 3);
  
  IntList_add(field[9][9], 4);
//  IntList_add(field[9][9], 4);
  IntList_add(field[9][8], 4);
  
  IntList_add(field[8][8], 5);
  IntList_add(field[7][8], 5);
  
  IntList_add(field[1][1], 9);
  
  animate = IntList_init();
  IntList_add(animate, 9);  // animate group 9
  
  // main loup
  while(done == 0) {   
    while ( SDL_PollEvent(&event) ) {
    
      if ( event.type == SDL_QUIT )  {  done = 1;  }     

      if ( event.type == SDL_KEYDOWN ) {
        if ( event.key.keysym.sym == SDLK_ESCAPE ) { done = 1; }              
      }
      
    } 
    
    SDL_FillRect(screen, NULL, 0);                   
    DrawScene(screen); 

    for (i = 0; i < animate->items; i++) {
      IntList_next(groups[animate->c[i]]);   //animate each group in list
    }                
    
    // limit refresh to 50Hz, for serial xmitt
    usleep(30000); 
 }




  for(i = 0; i < numtiles; i++) {
    SDL_FreeSurface(tiles[i]);
  }
    
  return 0;
}



/****************************/

void DrawScene(SDL_Surface *screen) {
 int x, y, wx, wy;
 int i, j, k;
 SDL_Rect tileLoc;
 SDL_Surface *tile;

  for (y = 0; y < numcols; y++) {
    for (x = 0; x < numrows; x++) {
      wx = (x * Tile_HalfW + tileOffsetX) - (y * Tile_HalfW) + hOffset;
      wy = (x * Tile_HalfH - tileOffsetY) + (y * Tile_HalfH) + vOffset;
      tileLoc.x = wx;
      tileLoc.y = wy;
      for(i = 0; i < field[x][y]->items; i++) {
      
         j = field[x][y]->c[i];
         tile = tiles[ groups[j]->c[groups[j]->cursor]];
                  
         tileLoc.x = (wx - tile->w/2);
         tileLoc.y = (wy - tile->h/2);
         SDL_BlitSurface(tile, NULL, screen, &tileLoc);
      }
    }
  }
  
 /*
  rectangleRGBA (screen, 0, 0, 256, 256, 250, 250, 250, 255 ); // navigation box
  stringRGBA(screen, 257, offset+1*(space+lineheight), " 4800N81",     0, 0, 255, 255); // foregroud colour
  circleRGBA (screen, 128, 128, x-1,  ((255-x)*2)-1, (x*2)-1, 0, 255 );
  lineRGBA (screen, px,    py-10, px,    py+10, 0, 0, 255, 255 );
*/
  
  

  SDL_Flip(screen);
  
}

/**************************************************************************/

void Init(void) {

  if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
    printf("Unable to init SDL: %s\n", SDL_GetError());
    exit(1);
  }
  atexit(SDL_Quit);

  
  screen=SDL_SetVideoMode(WIDTH, HEIGHT, 32, SDL_HWSURFACE|SDL_DOUBLEBUF);
//  screen=SDL_SetVideoMode(WIDTH, HEIGHT, 0, 0);
  
  if ( screen == NULL ) {
    printf("Unable to set up video: %s\n", SDL_GetError());
    exit(1);
  }
  
  //turn off cursor...
  //SDL_ShowCursor(0);
}







